@@ -1,47 +1,53 @@
-/*********************************************************************
- * Copyright (c) 2019-2020 Red Hat, Inc.
- *
+/**
+ * Copyright (c) 2019-2021 Red Hat, Inc.
  * This program and the accompanying materials are made
  * available under the terms of the Eclipse Public License 2.0
  * which is available at https://www.eclipse.org/legal/epl-2.0/
  *
  * SPDX-License-Identifier: EPL-2.0
- **********************************************************************/
+ *
+ * Contributors:
+ *   Red Hat, Inc. - initial API and implementation
+ */
 
 import { che as chetypes } from '@eclipse-che/api'
 import { CoreV1Api, V1Pod, Watch } from '@kubernetes/client-node'
 import axios, { AxiosInstance } from 'axios'
 import * as cp from 'child_process'
-import { cli } from 'cli-ux'
 import * as commandExists from 'command-exists'
 import * as fs from 'fs-extra'
 import * as https from 'https'
 import * as yaml from 'js-yaml'
+import * as nodeforge from 'node-forge'
 import * as os from 'os'
 import * as path from 'path'
-
+import * as rimraf from 'rimraf'
+import * as unzipper from 'unzipper'
 import { OpenShiftHelper } from '../api/openshift'
-import { CHE_ROOT_CA_SECRET_NAME, DEFAULT_CA_CERT_FILE_NAME } from '../constants'
-import { base64Decode } from '../util'
-
-import { Devfile } from './devfile'
+import { CHE_ROOT_CA_SECRET_NAME, DEFAULT_CHE_OLM_PACKAGE_NAME, DEFAULT_OPENSHIFT_OPERATORS_NS_NAME, OPERATOR_TEMPLATE_DIR } from '../constants'
+import { base64Decode, downloadFile } from '../util'
+import { CheApiClient } from './che-api-client'
+import { Devfile } from './types/devfile'
 import { KubeHelper } from './kube'
+import { OperatorGroup, Subscription } from './types/olm'
 
 export class CheHelper {
   defaultCheResponseTimeoutMs = 3000
+
   kube: KubeHelper
+
   oc = new OpenShiftHelper()
 
   private readonly axios: AxiosInstance
 
-  constructor(flags: any) {
+  constructor(private readonly flags: any) {
     this.kube = new KubeHelper(flags)
 
     // Make axios ignore untrusted certificate error for self-signed certificate case.
     const httpsAgent = new https.Agent({ rejectUnauthorized: false })
 
     this.axios = axios.create({
-      httpsAgent
+      httpsAgent,
     })
   }
 
@@ -51,7 +57,7 @@ export class CheHelper {
    * or if workspace ID wasn't specified but more than one workspace is found.
    */
   async getWorkspacePodName(namespace: string, cheWorkspaceId: string): Promise<string> {
-    const k8sApi = KubeHelper.KUBE_CONFIG.makeApiClient(CoreV1Api)
+    const k8sApi = this.kube.kubeConfig.makeApiClient(CoreV1Api)
 
     const res = await k8sApi.listNamespacedPod(namespace)
     const pods = res.body.items
@@ -75,7 +81,7 @@ export class CheHelper {
   }
 
   async getWorkspacePodContainers(namespace: string, cheWorkspaceId?: string): Promise<string[]> {
-    const k8sApi = KubeHelper.KUBE_CONFIG.makeApiClient(CoreV1Api)
+    const k8sApi = this.kube.kubeConfig.makeApiClient(CoreV1Api)
 
     const res = await k8sApi.listNamespacedPod(namespace)
     const pods = res.body.items
@@ -99,7 +105,7 @@ export class CheHelper {
   }
 
   async cheURL(namespace = ''): Promise<string> {
-    if (!await this.cheNamespaceExist(namespace)) {
+    if (!await this.kube.getNamespace(namespace)) {
       throw new Error(`ERR_NAMESPACE_NO_EXIST - No namespace ${namespace} is found`)
     }
 
@@ -110,9 +116,27 @@ export class CheHelper {
     }
   }
 
+  async chePluginRegistryURL(namespace = ''): Promise<string> {
+    // provided through command line ?
+    if (this.flags['plugin-registry-url']) {
+      return this.flags['plugin-registry-url']
+    }
+    // check
+    if (!await this.kube.getNamespace(namespace)) {
+      throw new Error(`ERR_NAMESPACE_NO_EXIST - No namespace ${namespace} is found`)
+    }
+
+    // grab URL
+    if (await this.kube.isOpenShift()) {
+      return this.chePluginRegistryOpenShiftURL(namespace)
+    } else {
+      return this.chePluginRegistryK8sURL(namespace)
+    }
+  }
+
   async isSelfSignedCertificateSecretExist(namespace: string): Promise<boolean> {
     const selfSignedCertSecret = await this.kube.getSecret(CHE_ROOT_CA_SECRET_NAME, namespace)
-    return !!selfSignedCertSecret
+    return Boolean(selfSignedCertSecret)
   }
 
   /**
@@ -120,6 +144,35 @@ export class CheHelper {
    * If secret doesn't exist, undefined is returned.
    */
   async retrieveCheCaCert(cheNamespace: string): Promise<string | undefined> {
+    const cheCaSecretContent = await this.getCheSelfSignedSecretContent(cheNamespace)
+    if (!cheCaSecretContent) {
+      return
+    }
+
+    const pemBeginHeader = '-----BEGIN CERTIFICATE-----'
+    const pemEndHeader = '-----END CERTIFICATE-----'
+    const certRegExp = new RegExp(`(^${pemBeginHeader}$(?:(?!${pemBeginHeader}).)*^${pemEndHeader}$)`, 'mgs')
+    const certsPem = cheCaSecretContent.match(certRegExp)
+
+    const caCertsPem: string[] = []
+    if (certsPem) {
+      for (const certPem of certsPem) {
+        const cert = nodeforge.pki.certificateFromPem(certPem)
+        const basicConstraintsExt = cert.getExtension('basicConstraints')
+        if (basicConstraintsExt && (basicConstraintsExt as any).cA) {
+          caCertsPem.push(certPem)
+        }
+      }
+    }
+
+    return caCertsPem.join('\n')
+  }
+
+  /**
+   * Retrieves content of Che self-signed-certificate secret or undefined if the secret doesn't exist.
+   * Note, it contains certificate chain in pem format.
+   */
+  private async getCheSelfSignedSecretContent(cheNamespace: string): Promise<string | undefined> {
     const cheCaSecret = await this.kube.getSecret(CHE_ROOT_CA_SECRET_NAME, cheNamespace)
     if (!cheCaSecret) {
       return
@@ -132,22 +185,8 @@ export class CheHelper {
     throw new Error(`Secret "${CHE_ROOT_CA_SECRET_NAME}" has invalid format: "ca.crt" key not found in data.`)
   }
 
-  async saveCheCaCert(cheCaCert: string, destinaton?: string): Promise<string> {
-    if (destinaton && fs.existsSync(destinaton)) {
-      if (fs.lstatSync(destinaton).isDirectory()) {
-        destinaton = path.join(destinaton, DEFAULT_CA_CERT_FILE_NAME)
-      }
-    } else {
-      // Fallback to default location
-      destinaton = path.join(os.homedir(), DEFAULT_CA_CERT_FILE_NAME)
-    }
-
-    fs.writeFileSync(destinaton, cheCaCert)
-    return destinaton
-  }
-
   /**
-   * Retreives Keycloak admin user credentials.
+   * Retrieves Keycloak admin user credentials.
    * Works only with installers which use Che CR (operator, olm).
    * Returns credentials as an array of two values: [login, password]
    * In case of an error an array with undefined values will be returned.
@@ -157,9 +196,10 @@ export class CheHelper {
     let adminPassword
 
     const cheCluster = await this.kube.getCheCluster(cheNamespace)
-    if (!cheCluster) {
+    if (!cheCluster || cheCluster.spec.auth.externalIdentityProvider) {
       return []
     }
+
     const keycloakCredentialsSecretName = cheCluster.spec.auth.identityProviderSecret
     if (keycloakCredentialsSecretName) {
       // Keycloak credentials are stored in secret
@@ -177,10 +217,28 @@ export class CheHelper {
     return [adminUsername, adminPassword]
   }
 
+  async chePluginRegistryK8sURL(namespace = ''): Promise<string> {
+    if (await this.kube.isIngressExist('plugin-registry', namespace)) {
+      const protocol = await this.kube.getIngressProtocol('plugin-registry', namespace)
+      const hostname = await this.kube.getIngressHost('plugin-registry', namespace)
+      return `${protocol}://${hostname}`
+    }
+    throw new Error(`ERR_INGRESS_NO_EXIST - No ingress 'plugin-registry' in namespace ${namespace}`)
+  }
+
+  async chePluginRegistryOpenShiftURL(namespace = ''): Promise<string> {
+    if (await this.oc.routeExist('plugin-registry', namespace)) {
+      const protocol = await this.oc.getRouteProtocol('plugin-registry', namespace)
+      const hostname = await this.oc.getRouteHost('plugin-registry', namespace)
+      return `${protocol}://${hostname}`
+    }
+    throw new Error(`ERR_ROUTE_NO_EXIST - No route 'plugin-registry' in namespace ${namespace}`)
+  }
+
   async cheK8sURL(namespace = ''): Promise<string> {
     const ingress_names = ['che', 'che-ingress']
     for (const ingress_name of ingress_names) {
-      if (await this.kube.ingressExist(ingress_name, namespace)) {
+      if (await this.kube.isIngressExist(ingress_name, namespace)) {
         const protocol = await this.kube.getIngressProtocol(ingress_name, namespace)
         const hostname = await this.kube.getIngressHost(ingress_name, namespace)
         return `${protocol}://${hostname}`
@@ -201,170 +259,23 @@ export class CheHelper {
     throw new Error(`ERR_ROUTE_NO_EXIST - No route ${route_names} in namespace ${namespace}`)
   }
 
-  async cheNamespaceExist(namespace = '') {
-    return this.kube.namespaceExist(namespace)
-  }
-
-  async getCheServerStatus(cheURL: string, responseTimeoutMs = this.defaultCheResponseTimeoutMs): Promise<string> {
-    const endpoint = `${cheURL}/api/system/state`
-    let response = null
-    try {
-      response = await this.axios.get(endpoint, { timeout: responseTimeoutMs })
-    } catch (error) {
-      throw this.getCheApiError(error, endpoint)
-    }
-    if (!response || response.status !== 200 || !response.data || !response.data.status) {
-      throw new Error('E_BAD_RESP_CHE_API')
-    }
-    return response.data.status
-  }
-
-  async startShutdown(cheURL: string, accessToken = '', responseTimeoutMs = this.defaultCheResponseTimeoutMs) {
-    const endpoint = `${cheURL}/api/system/stop?shutdown=true`
-    const headers = accessToken ? { Authorization: `${accessToken}` } : null
-    let response = null
-    try {
-      response = await this.axios.post(endpoint, null, { headers, timeout: responseTimeoutMs })
-    } catch (error) {
-      if (error.response && error.response.status === 409) {
-        return
-      } else {
-        throw this.getCheApiError(error, endpoint)
-      }
-    }
-    if (!response || response.status !== 204) {
-      throw new Error('E_BAD_RESP_CHE_API')
-    }
-  }
-
-  async waitUntilReadyToShutdown(cheURL: string, intervalMs = 500, timeoutMs = 60000) {
-    const iterations = timeoutMs / intervalMs
-    for (let index = 0; index < iterations; index++) {
-      let status = await this.getCheServerStatus(cheURL)
-      if (status === 'READY_TO_SHUTDOWN') {
-        return
-      }
-      await cli.wait(intervalMs)
-    }
-    throw new Error('ERR_TIMEOUT')
-  }
-
-  async isCheServerReady(cheURL: string, responseTimeoutMs = this.defaultCheResponseTimeoutMs): Promise<boolean> {
-    const id = await this.axios.interceptors.response.use(response => response, async (error: any) => {
-      if (error.config && error.response && (error.response.status === 404 || error.response.status === 503)) {
-        return this.axios.request(error.config)
-      }
-      return Promise.reject(error)
-    })
-
-    try {
-      await this.axios.get(`${cheURL}/api/system/state`, { timeout: responseTimeoutMs })
-      await this.axios.interceptors.response.eject(id)
-      return true
-    } catch {
-      await this.axios.interceptors.response.eject(id)
-      return false
-    }
-  }
-
-  async startWorkspace(cheNamespace: string, workspaceId: string, debug: boolean, accessToken: string | undefined): Promise<void> {
-    const cheUrl = await this.cheURL(cheNamespace)
-    let endpoint = `${cheUrl}/api/workspace/${workspaceId}/runtime`
-    if (debug) {
-      endpoint += '?debug-workspace-start=true'
-    }
-    let response
-
-    const headers: { [key: string]: string } = {}
-    if (accessToken) {
-      headers.Authorization = accessToken
-    }
-    try {
-      response = await this.axios.post(endpoint, undefined, { headers })
-    } catch (error) {
-      if (error.response && error.response.status === 404) {
-        throw new Error(`E_WORKSPACE_NOT_EXIST - workspace with "${workspaceId}" id doesn't exist`)
-      } else {
-        throw this.getCheApiError(error, endpoint)
-      }
-    }
-
-    if (!response || response.status !== 200 || !response.data) {
-      throw new Error('E_BAD_RESP_CHE_API')
-    }
-  }
-
-  async stopWorkspace(cheUrl: string, workspaceId: string, accessToken?: string): Promise<void> {
-    let endpoint = `${cheUrl}/api/workspace/${workspaceId}/runtime`
-    let response
-
-    const headers: { [key: string]: string } = {}
-    if (accessToken) {
-      headers.Authorization = accessToken
-    }
-    try {
-      response = await this.axios.delete(endpoint, { headers })
-    } catch (error) {
-      if (error.response && error.response.status === 404) {
-        throw new Error(`E_WORKSPACE_NOT_EXIST - workspace with "${workspaceId}" id doesn't exist`)
-      } else {
-        throw this.getCheApiError(error, endpoint)
-      }
-    }
-
-    if (!response || response.status !== 204) {
-      throw new Error('E_BAD_RESP_CHE_API')
-    }
-  }
-
-  async createWorkspaceFromDevfile(namespace: string | undefined, devfilePath = '', workspaceName: string | undefined, accessToken = ''): Promise<chetypes.workspace.Workspace> {
-    if (!await this.cheNamespaceExist(namespace)) {
-      throw new Error('E_BAD_NS')
-    }
-    let url = await this.cheURL(namespace)
-    let endpoint = `${url}/api/workspace/devfile`
+  async createWorkspaceFromDevfile(cheApiEndpoint: string, devfilePath: string, workspaceName?: string, accessToken?: string): Promise<chetypes.workspace.Workspace> {
     let devfile: string | undefined
-    let response
-    const headers: any = { 'Content-Type': 'text/yaml' }
-    if (accessToken && accessToken.length > 0) {
-      headers.Authorization = `${accessToken}`
-    }
-
     try {
       devfile = await this.parseDevfile(devfilePath)
       if (workspaceName) {
-        let json: Devfile = yaml.load(devfile)
+        const json = yaml.load(devfile) as Devfile
         json.metadata.name = workspaceName
         devfile = yaml.dump(json)
       }
-
-      response = await this.axios.post(endpoint, devfile, { headers })
     } catch (error) {
       if (!devfile) {
         throw new Error(`E_NOT_FOUND_DEVFILE - ${devfilePath} - ${error.message}`)
       }
-
-      if (error.response) {
-        if (error.response.status === 400) {
-          throw new Error(`E_BAD_DEVFILE_FORMAT - Message: ${error.response.data.message}`)
-        }
-        if (error.response.status === 409) {
-          let message = ''
-          if (error.response.data) {
-            message = error.response.data.message
-          }
-          throw new Error(`E_CONFLICT - Message: ${message}`)
-        }
-      }
-
-      throw this.getCheApiError(error, endpoint)
     }
 
-    if (response && response.data) {
-      return response.data as chetypes.workspace.Workspace
-    } else {
-      throw new Error('E_BAD_RESP_CHE_SERVER')
-    }
+    const cheApi = CheApiClient.getInstance(cheApiEndpoint)
+    return cheApi.createWorkspaceFromDevfile(devfile, accessToken)
   }
 
   async parseDevfile(devfilePath = ''): Promise<string> {
@@ -376,24 +287,6 @@ export class CheHelper {
     }
   }
 
-  async isAuthenticationEnabled(cheURL: string, responseTimeoutMs = this.defaultCheResponseTimeoutMs): Promise<boolean> {
-    const endpoint = `${cheURL}/api/keycloak/settings`
-    let response = null
-    try {
-      response = await this.axios.get(endpoint, { timeout: responseTimeoutMs })
-    } catch (error) {
-      if (error.response && (error.response.status === 404 || error.response.status === 503)) {
-        return false
-      } else {
-        throw this.getCheApiError(error, endpoint)
-      }
-    }
-    if (!response || response.status !== 200 || !response.data) {
-      throw new Error('E_BAD_RESP_CHE_API')
-    }
-    return true
-  }
-
   async buildDashboardURL(ideURL: string): Promise<string> {
     return ideURL.replace(/\/[^/|.]*\/[^/|.]*$/g, '\/dashboard\/#\/ide$&')
   }
@@ -401,27 +294,11 @@ export class CheHelper {
   /**
    * Finds workspace pods and reads logs from it.
    */
-  async readWorkspacePodLog(namespace: string, workspaceId: string, directory: string): Promise<boolean> {
+  async readWorkspacePodLog(namespace: string, workspaceId: string, directory: string, follow: boolean): Promise<void> {
     const podLabelSelector = `che.workspace_id=${workspaceId}`
 
-    let workspaceIsRun = false
-
-    const pods = await this.kube.listNamespacedPod(namespace, undefined, podLabelSelector)
-    if (pods.items.length) {
-      workspaceIsRun = true
-    }
-
-    for (const pod of pods.items) {
-      for (const containerStatus of pod.status!.containerStatuses!) {
-        workspaceIsRun = workspaceIsRun && !!containerStatus.state && !!containerStatus.state.running
-      }
-    }
-
-    const follow = !workspaceIsRun
     await this.readPodLog(namespace, podLabelSelector, directory, follow)
     await this.readNamespaceEvents(namespace, directory, follow)
-
-    return workspaceIsRun
   }
 
   /**
@@ -476,8 +353,8 @@ export class CheHelper {
   async watchNamespacedPods(namespace: string, podLabelSelector: string | undefined, directory: string): Promise<void> {
     const processedContainers = new Map<string, Set<string>>()
 
-    const watcher = new Watch(KubeHelper.KUBE_CONFIG)
-    watcher.watch(`/api/v1/namespaces/${namespace}/pods`, {},
+    const watcher = new Watch(this.kube.kubeConfig)
+    return watcher.watch(`/api/v1/namespaces/${namespace}/pods`, {},
       async (_phase: string, obj: any) => {
         const pod = obj as V1Pod
         if (!pod || !pod.metadata || !pod.metadata.name) {
@@ -505,85 +382,6 @@ export class CheHelper {
       () => { })
   }
 
-  async getAllWorkspaces(cheURL: string, accessToken?: string): Promise<any[]> {
-    const all: any[] = []
-    const maxItems = 30
-    let skipCount = 0
-
-    do {
-      const workspaces = await this.doGetWorkspaces(cheURL, skipCount, maxItems, accessToken)
-      all.push(...workspaces)
-      skipCount += workspaces.length
-    } while (all.length === maxItems)
-
-    return all
-  }
-
-  /**
-   * Returns list of workspaces
-   */
-  async doGetWorkspaces(cheUrl: string, skipCount: number, maxItems: number, accessToken = ''): Promise<any[]> {
-    const endpoint = `${cheUrl}/api/workspace?skipCount=${skipCount}&maxItems=${maxItems}`
-    const headers: any = { 'Content-Type': 'text/yaml' }
-    if (accessToken && accessToken.length > 0) {
-      headers.Authorization = `${accessToken}`
-    }
-
-    try {
-      const response = await this.axios.get(endpoint, { headers })
-      if (response && response.data) {
-        return response.data
-      } else {
-        throw new Error('E_BAD_RESP_CHE_SERVER')
-      }
-    } catch (error) {
-      throw this.getCheApiError(error, endpoint)
-    }
-  }
-
-  /**
-   * Get workspace.
-   */
-  async getWorkspace(cheUrl: string, workspaceId: string, accessToken = ''): Promise<any> {
-    const endpoint = `${cheUrl}/api/workspace/${workspaceId}`
-    const headers: any = { 'Content-Type': 'text/yaml' }
-    if (accessToken) {
-      headers.Authorization = `${accessToken}`
-    }
-
-    try {
-      const response = await this.axios.get(endpoint, { headers })
-      return response.data
-    } catch (error) {
-      if (error.response.status === 404) {
-        throw new Error(`Workspace ${workspaceId} not found. Please use the command workspace:list to get list of the existed workspaces.`)
-      }
-      throw this.getCheApiError(error, endpoint)
-    }
-  }
-
-  /**
-   * Deletes workspace.
-   */
-  async deleteWorkspace(cheUrl: string, workspaceId: string, accessToken = ''): Promise<void> {
-    const endpoint = `${cheUrl}/api/workspace/${workspaceId}`
-    const headers: any = {}
-    if (accessToken) {
-      headers.Authorization = `${accessToken}`
-    }
-
-    try {
-      await this.axios.delete(endpoint, { headers })
-    } catch (error) {
-      if (error.response.status === 404) {
-        throw new Error(`Workspace ${workspaceId} not found. Please use the command workspace:list to get list of the existed workspaces.`)
-      } else if (error.response.status === 409) {
-        throw new Error('Cannot delete a running workspace. Please stop it using the command workspace:stop and try again')
-      }
-      throw this.getCheApiError(error, endpoint)
-    }
-  }
-
   /**
    * Indicates if pod matches given labels.
    */
@@ -634,29 +432,126 @@ export class CheHelper {
     return fileName
   }
 
-  getCheApiError(error: any, endpoint: string): Error {
-    if (error.response) {
-      const status = error.response.status
-      if (status === 403) {
-        return new Error(`E_CHE_API_FORBIDDEN - Endpoint: ${endpoint} - Message: ${JSON.stringify(error.response.data.message)}`)
-      } else if (status === 401) {
-        return new Error(`E_CHE_API_UNAUTHORIZED - Endpoint: ${endpoint} - Message: ${JSON.stringify(error.response.data)}`)
-      } else if (status === 404) {
-        return new Error(`E_CHE_API_NOTFOUND - Endpoint: ${endpoint} - Message: ${JSON.stringify(error.response.data)}`)
-      } else {
-        // The request was made and the server responded with a status code
-        // that falls out of the range of 2xx
-        return new Error(`E_CHE_API_UNKNOWN_ERROR - Endpoint: ${endpoint} -Status: ${error.response.status}`)
-      }
+  /**
+   * Gets install templates for given installer.
+   * @param installer Che installer
+   * @param url link to zip archive with sources of Che operator
+   * @param destDir destination directory into which the templates should be unpacked
+   */
+  async downloadAndUnpackTemplates(installer: string, url: string, destDir: string): Promise<void> {
+    // Add codeready-operator folder for operator templates
+    if (installer === 'operator') {
+      destDir = path.join(destDir, OPERATOR_TEMPLATE_DIR)
+    }
 
-    } else if (error.request) {
-      // The request was made but no response was received
-      // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
-      // http.ClientRequest in node.js
-      return new Error(`E_CHE_API_NO_RESPONSE - Endpoint: ${endpoint} - Error message: ${error.message}`)
-    } else {
-      // Something happened in setting up the request that triggered an Error
-      return new Error(`E_CHECTL_UNKNOWN_ERROR - Endpoint: ${endpoint} - Message: ${error.message}`)
+    const tempDir = path.join(os.tmpdir(), Date.now().toString())
+    await fs.mkdirp(tempDir)
+    const zipFile = path.join(tempDir, `che-templates-${installer}.zip`)
+    await downloadFile(url, zipFile)
+    await this.unzipTemplates(zipFile, destDir)
+    // Clean up zip. Do not wait when finishes.
+    rimraf(tempDir, () => { })
+  }
+
+  /**
+   * Unpacks repository deploy templates into specified folder
+   * @param zipFile path to zip archive with source code
+   * @param destDir target directory into which templates should be unpacked
+   */
+  private async unzipTemplates(zipFile: string, destDir: string) {
+    // Gets path from: repo-name/deploy/path
+    const deployDirRegex = new RegExp('(?:^[\\\w-]*\\\/deploy\\\/)(.*)')
+    const configDirRegex = new RegExp('(?:^[\\\w-]*\\\/config\\\/)(.*)')
+
+    const zip = fs.createReadStream(zipFile).pipe(unzipper.Parse({ forceStream: true }))
+    for await (const entry of zip) {
+      const entryPathInZip: string = entry.path
+      const templatesPathMatch = entryPathInZip.match(deployDirRegex) || entryPathInZip.match(configDirRegex)
+      if (templatesPathMatch && templatesPathMatch.length > 1 && templatesPathMatch[1]) {
+        // Remove prefix from in-zip path
+        const entryPathWhenExtracted = templatesPathMatch[1]
+        // Path to the item in target location
+        const dest = path.join(destDir, entryPathWhenExtracted)
+
+        // Extract item
+        if (entry.type === 'File') {
+          const parentDirName = path.dirname(dest)
+          if (!fs.existsSync(parentDirName)) {
+            await fs.mkdirp(parentDirName)
+          }
+          entry.pipe(fs.createWriteStream(dest))
+        } else if (entry.type === 'Directory') {
+          if (!fs.existsSync(dest)) {
+            await fs.mkdirp(dest)
+          }
+          // The folder is created above
+          entry.autodrain()
+        } else {
+          // Ignore the item as we do not need to handle links and etc.
+          entry.autodrain()
+        }
+      } else {
+        // No need to extract this item
+        entry.autodrain()
+      }
+    }
+
+    // Is a new project structure?
+    if (fs.existsSync(path.join(destDir, 'manager', 'manager.yaml'))) {
+      fs.moveSync(path.join(destDir, 'manager', 'manager.yaml'), path.join(destDir, 'operator.yaml'))
+      fs.moveSync(path.join(destDir, 'rbac', 'service_account.yaml'), path.join(destDir, 'service_account.yaml'))
+      fs.moveSync(path.join(destDir, 'rbac', 'role.yaml'), path.join(destDir, 'role.yaml'))
+      fs.moveSync(path.join(destDir, 'rbac', 'role_binding.yaml'), path.join(destDir, 'role_binding.yaml'))
+      fs.moveSync(path.join(destDir, 'rbac', 'cluster_role.yaml'), path.join(destDir, 'cluster_role.yaml'))
+      fs.moveSync(path.join(destDir, 'rbac', 'cluster_rolebinding.yaml'), path.join(destDir, 'cluster_rolebinding.yaml'))
+      fs.moveSync(path.join(destDir, 'crd', 'bases'), path.join(destDir, 'crds'))
+      fs.moveSync(path.join(destDir, 'samples', 'org.eclipse.che_v1_checluster.yaml'), path.join(destDir, 'crds', 'org_v1_che_cr.yaml'))
+      fs.moveSync(path.join(destDir, 'samples', 'org_v1_chebackupserverconfiguration.yaml'), path.join(destDir, 'crds', 'org.eclipse.che_v1_chebackupserverconfiguration_cr.yaml'))
+      fs.moveSync(path.join(destDir, 'samples', 'org_v1_checlusterbackup.yaml'), path.join(destDir, 'crds', 'org.eclipse.che_v1_checlusterbackup_cr.yaml'))
+      fs.moveSync(path.join(destDir, 'samples', 'org_v1_checlusterrestore.yaml'), path.join(destDir, 'crds', 'org.eclipse.che_v1_checlusterrestore_cr.yaml'))
     }
   }
+
+  async findCheOperatorSubscription(namespace: string): Promise<Subscription | undefined> {
+    try {
+      const subscriptions = await this.kube.listOperatorSubscriptions(namespace)
+      const cheSubscriptions = subscriptions.filter(subscription => subscription.spec.name && subscription.spec.name.includes(DEFAULT_CHE_OLM_PACKAGE_NAME))
+      if (cheSubscriptions.length > 1) {
+        throw new Error('Found more than one Che subscription')
+      }
+      if (cheSubscriptions.length === 1) {
+        return cheSubscriptions[0]
+      }
+      // No subscriptions found, check if Che is installed in all namespaces mode
+      if (namespace !== DEFAULT_OPENSHIFT_OPERATORS_NS_NAME) {
+        return this.findCheOperatorSubscription(DEFAULT_OPENSHIFT_OPERATORS_NS_NAME)
+      }
+    } catch {
+      // Do nothing, just return undefined
+    }
+  }
+
+  async findCheOperatorOperatorGroup(namespace: string): Promise<OperatorGroup | undefined> {
+    const subscription = await this.findCheOperatorSubscription(namespace)
+    if (!subscription || !subscription.status || !subscription.status.installedCSV) {
+      return
+    }
+
+    const csvName = subscription.status.installedCSV
+    if (subscription.metadata.namespace) {
+      namespace = subscription.metadata.namespace
+    }
+    const csv = await this.kube.getCSV(csvName, namespace)
+    if (!csv || !csv.metadata || !csv.metadata.annotations) {
+      return
+    }
+
+    const operatorGroupName = csv.metadata.annotations['olm.operatorGroup']
+    const operatorGroupNamespace = csv.metadata.annotations['olm.operatorNamespace']
+    if (!operatorGroupName || !operatorGroupNamespace) {
+      return
+    }
+
+    return this.kube.getOperatorGroup(operatorGroupName, operatorGroupNamespace)
+  }
 }
