@@ -1,12 +1,14 @@
-/*********************************************************************
- * Copyright (c) 2019 Red Hat, Inc.
- *
+/**
+ * Copyright (c) 2019-2021 Red Hat, Inc.
  * This program and the accompanying materials are made
  * available under the terms of the Eclipse Public License 2.0
  * which is available at https://www.eclipse.org/legal/epl-2.0/
  *
  * SPDX-License-Identifier: EPL-2.0
- **********************************************************************/
+ *
+ * Contributors:
+ *   Red Hat, Inc. - initial API and implementation
+ */
 
 import { Command, flags } from '@oclif/command'
 import { boolean } from '@oclif/command/lib/flags'
@@ -23,10 +25,10 @@ import { DevWorkspaceTasks } from '../../tasks/component-installers/devfile-work
 import { OLMTasks } from '../../tasks/installers/olm'
 import { OperatorTasks } from '../../tasks/installers/operator'
 import { ApiTasks } from '../../tasks/platforms/api'
-import { findWorkingNamespace, getCommandErrorMessage, getCommandSuccessMessage, notifyCommandCompletedSuccessfully } from '../../util'
+import { findWorkingNamespace, getCommandSuccessMessage, notifyCommandCompletedSuccessfully, wrapCommandError } from '../../util'
 
 export default class Delete extends Command {
-  static description = 'delete any CodeReady Workspaces related resource'
+  static description = 'delete any CodeReady Workspaces related resource: Kubernetes/OpenShift'
 
   static flags: flags.Input<any> = {
     help: flags.help({ char: 'h' }),
@@ -34,7 +36,7 @@ export default class Delete extends Command {
     batch,
     'delete-namespace': boolean({
       description: 'Indicates that a CodeReady Workspaces namespace will be deleted as well',
-      default: false
+      default: false,
     }),
     'deployment-name': cheDeployment,
     'listr-renderer': listrRenderer,
@@ -45,7 +47,7 @@ export default class Delete extends Command {
     }),
     'skip-kubernetes-health-check': skipKubeHealthzCheck,
     yes: assumeYes,
-    telemetry: CHE_TELEMETRY
+    telemetry: CHE_TELEMETRY,
   }
 
   async run() {
@@ -69,7 +71,7 @@ export default class Delete extends Command {
     const devWorkspaceTasks = new DevWorkspaceTasks(flags)
 
     const tasks = new Listrq([], ctx.listrOptions)
-    tasks.add(apiTasks.testApiTasks(flags, this))
+    tasks.add(apiTasks.testApiTasks(flags))
     tasks.add(operatorTasks.deleteTasks(flags))
     tasks.add(olmTasks.deleteTasks(flags))
     tasks.add(cheTasks.deleteTasks(flags))
@@ -84,7 +86,8 @@ export default class Delete extends Command {
           return new Listr(devWorkspaceTasks.getUninstallTasks())
         }
         task.title = `${task.title}...Skipped: another CodeReady Workspaces deployment found.`
-      }})
+      },
+    })
 
     if (flags['delete-namespace']) {
       tasks.add(cheTasks.deleteNamespace(flags))
@@ -95,13 +98,15 @@ export default class Delete extends Command {
         await tasks.run()
         cli.log(getCommandSuccessMessage())
       } catch (err) {
-        this.error(getCommandErrorMessage(err))
+        this.error(wrapCommandError(err))
       }
     } else {
       this.exit(0)
     }
 
-    notifyCommandCompletedSuccessfully()
+    if (!flags.batch) {
+      notifyCommandCompletedSuccessfully()
+    }
     this.exit(0)
   }
 
