@@ -9,6 +9,7 @@
  * Contributors:
  *   Red Hat, Inc. - initial API and implementation
  */
+
 import { V1ClusterRole, V1ClusterRoleBinding, V1Deployment, V1Role, V1RoleBinding } from '@kubernetes/client-node'
 import { Command } from '@oclif/command'
 import { cli } from 'cli-ux'
@@ -90,7 +91,7 @@ export class OperatorTasks {
         }
 
         for (const role of ctx.roles as V1Role[]) {
-          if (await kube.roleExist(role.metadata!.name, flags.chenamespace)) {
+          if (await kube.isRoleExist(role.metadata!.name, flags.chenamespace)) {
             if (shouldUpdate) {
               await kube.replaceRoleFromObj(role, flags.chenamespace)
             }
@@ -100,7 +101,7 @@ export class OperatorTasks {
         }
 
         for (const roleBinding of ctx.roleBindings as V1RoleBinding[]) {
-          if (await kube.roleBindingExist(roleBinding.metadata!.name, flags.chenamespace)) {
+          if (await kube.isRoleBindingExist(roleBinding.metadata!.name, flags.chenamespace)) {
             if (shouldUpdate) {
               await kube.replaceRoleBindingFromObj(roleBinding, flags.chenamespace)
             }
