@@ -21,12 +21,12 @@ import * as yaml from 'js-yaml'
 import * as notifier from 'node-notifier'
 import * as os from 'os'
 import * as path from 'path'
+import * as readline from 'readline'
 import { promisify } from 'util'
-
 import { ChectlContext } from './api/context'
 import { KubeHelper } from './api/kube'
 import { VersionHelper } from './api/version'
-import { DEFAULT_CHE_NAMESPACE, LEGACY_CHE_NAMESPACE } from './constants'
+import { DEFAULT_CHE_NAMESPACE, DEFAULT_CHE_TLS_SECRET_NAME, LEGACY_CHE_NAMESPACE } from './constants'
 
 const pkjson = require('../package.json')
 
@@ -85,6 +85,10 @@ export function base64Decode(arg: string): string {
   return Buffer.from(arg, 'base64').toString('ascii')
 }
 
+export function base64Encode(arg: string): string {
+  return Buffer.from(arg).toString('base64')
+}
+
 /**
  * Separates docker image repository and tag.
  * @param image string with image and tag separated by a colon
@@ -255,11 +259,11 @@ export function readPackageJson(): any {
 }
 
 export function safeLoadFromYamlFile(filePath: string): any {
-  return yaml.safeLoad(fs.readFileSync(filePath).toString())
+  return yaml.load(fs.readFileSync(filePath).toString())
 }
 
 export function safeSaveYamlToFile(yamlObject: any, filePath: string): void {
-  fs.writeFileSync(filePath, yaml.safeDump(yamlObject))
+  fs.writeFileSync(filePath, yaml.dump(yamlObject))
 }
 
 export async function downloadFile(url: string, dest: string): Promise<void> {
@@ -281,7 +285,7 @@ export async function downloadYaml(url: string): Promise<any> {
     httpsAgent: new https.Agent({}),
   })
   const response = await axiosInstance.get(url)
-  return yaml.safeLoad(response.data)
+  return yaml.load(response.data)
 }
 
 export function getEmbeddedTemplatesDirectory(): string {
@@ -338,3 +342,107 @@ export async function getDistribution(): Promise<string | undefined> {
   }
   return
 }
+
+export function addTrailingSlash(url: string): string {
+  if (url.endsWith('/')) {
+    return url
+  }
+  return url + '/'
+}
+
+/**
+ * Waits until y or n is pressed (no return press needed) and returns confirmation result.
+ * ctrl+c causes exception.
+ * This function doesn't print anything, this is the task of the code that invokes it.
+ */
+export function confirmYN(): Promise<boolean> {
+  return new Promise((resolve, reject) => {
+    readline.emitKeypressEvents(process.stdin)
+    if (process.stdin.isTTY) {
+      process.stdin.setRawMode(true)
+    }
+
+    const removeKeyPressHandler = () => {
+      process.stdin.removeListener('keypress', keyPressHandler)
+      process.stdin.setRawMode(false)
+      process.stdin.destroy()
+    }
+    const keyPressHandler = (_string: any, key: any) => {
+      // Handle brake
+      if (key.ctrl && key.name === 'c') {
+        removeKeyPressHandler()
+        reject('Interrupted')
+      }
+
+      // Check if y or n pressed
+      if (key.name === 'y' || key.name === 'Y') {
+        removeKeyPressHandler()
+        resolve(true)
+      } else if (key.name === 'n' || key.name === 'N') {
+        removeKeyPressHandler()
+        resolve(false)
+      }
+    }
+    process.stdin.on('keypress', keyPressHandler)
+  })
+}
+
+/**
+ * Checks if TLS is disabled via operator custom resource.
+ * Returns true if TLS is enabled (or omitted) and false if it is explicitly disabled.
+ */
+export function getTlsSupport(ctx: any): boolean {
+  const crPatch = ctx[ChectlContext.CR_PATCH]
+  if (crPatch && crPatch.spec && crPatch.spec.server && crPatch.spec.server.tlsSupport === false) {
+    return false
+  }
+
+  const customCR = ctx.customCR
+  if (customCR && customCR.spec && customCR.spec.server && customCR.spec.server.tlsSupport === false) {
+    return false
+  }
+
+  return true
+}
+
+export function isDevWorkspaceEnabled(ctx: any): boolean {
+  const crPatch = ctx[ChectlContext.CR_PATCH]
+  if (crPatch && crPatch.spec && crPatch.spec.devWorkspace && crPatch.spec.devWorkspace.enable) {
+    return true
+  }
+
+  const customCR = ctx.customCR
+  if (customCR && customCR.spec && customCR.spec.devWorkspace && customCR.spec.devWorkspace.enable) {
+    return true
+  }
+
+  return false
+}
+
+export function isNativeUserModeEnabled(ctx: any): boolean {
+  const crPatch = ctx[ChectlContext.CR_PATCH]
+  if (crPatch && crPatch.spec && crPatch.spec.auth && crPatch.spec.auth.nativeUserMode) {
+    return true
+  }
+
+  const customCR = ctx.customCR
+  if (customCR && customCR.spec && customCR.spec.auth && customCR.spec.auth.nativeUserMode) {
+    return true
+  }
+
+  return false
+}
+
+export function getTlsSecretName(ctx: any): string {
+  const crPatch = ctx[ChectlContext.CR_PATCH]
+  if (crPatch && crPatch.spec && crPatch.spec.k8s && crPatch.spec.k8s.tlsSecretName) {
+    return crPatch.spec.k8s.tlsSecretName
+  }
+
+  const customCR = ctx.customCR
+  if (customCR && customCR.spec && customCR.spec.k8s && customCR.spec.k8s.tlsSecretName) {
+    return customCR.spec.k8s.tlsSecretName
+  }
+
+  return DEFAULT_CHE_TLS_SECRET_NAME
+}
